//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#include <vcl\classes.hpp>
#include <vcl\sysutils.hpp>
#include <vcl\clipbrd.hpp>
#include <vcl\system.hpp>
#include <stdio.h>
#include <stdlib.h>
#include <dir.h>
#include <dos.h>
#include <fcntl.h>
#include <io.h>
#include <win32\shellapi.h>
#pragma hdrstop

#include "ImageKit.h"
#include "ImgkDef.h"

#include "MdiList.h"
#include "Timer.h"
#include "MdiFrame.h"
#include "MdiView.h"
#include "Text.h"
#include "DirectoryList.h"
#include "ListSize.h"
#include "OverView.h"
#include "Histgram.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TListForm *ListForm;
//---------------------------------------------------------------------------
__fastcall TListForm::TListForm(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TListForm::FormCreate(TObject *Sender)
{
    int pos;
    int filter;

    ListSize = FrameForm->RListSize;
    MaxImageWidth = FrameForm->RMaxImageWidth;
    ListColumn = MaxImageWidth / FaceWidth;

	ListBitmap = new Graphics::TBitmap;
    ListFiles = new TStringList;
    ListSizes = new TStringList;
    ListComments = new TStringList;
    ListIndex = new TStringList;

    ListBitmap->Width = FaceWidth;
    ListBitmap->Height = FaceHeight;

    ListFiles->Sorted = true;
    ListFiles->Duplicates = dupIgnore;

    ListSizes->Sorted = false;
    ListSizes->Duplicates = dupAccept;

    ListComments->Sorted = false;
    ListComments->Duplicates = dupAccept;

    ListIndex->Sorted = false;
    ListIndex->Duplicates = dupAccept;

    ListDir = FrameForm->OpenListDialog->FileName;
    pos = ListDir.Pos(ExtractFileName(ListDir));
    ListDir.SetLength(pos - 1);
    filter = FrameForm->OpenListDialog->FilterIndex;
    if (filter == 1)
    	ListFilter = "*.bmp *.rle";
    else if (filter == 2)
    	ListFilter = "*.jpg";
    else if (filter == 3)
    	ListFilter = "*.tif";
    else if (filter == 4)
    	ListFilter = "*.ico";
    else if (filter == 5)
    	ListFilter = "*.cur";
    else if (filter == 6)
    	ListFilter = "*.wmf *.emf";
    else if (filter == 7)
    	ListFilter = "*.exe *.dll";
    else if (filter == 8)
    	ListFilter = "*.*";

    if (FrameForm->RListSquare == 0)
        Square1->Checked = false;
    else if (FrameForm->RListSquare == 1)
        Square1->Checked = true;

    ListCount = 0;
    Target = 0;
    MouseTarget = 0;
    SelectCount = 0;
    Tag = 0;

    GetListFiles();

    ListFiles->Sorted = false;
}
//---------------------------------------------------------------------------
void __fastcall TListForm::GetListFiles(void)
{
    int i, j, k;
    int len;
    int done;
    int fsize, ksize;
    struct ffblk ffb;
    AnsiString SearchPath;
    AnsiString Filter;
    AnsiString FileName;
    AnsiString FileSize;

    ListCount = 0;
    i = 1;
    len = ListFilter.Length();
    Filter = ListFilter;

    while (1) {
    	j = Filter.Pos(" ");
        if (j != 0)
        	Filter = Filter.SubString(1, j - 1);

    	SearchPath = ListDir + Filter;

    	done = findfirst(SearchPath.c_str(), &ffb, FA_ARCH | FA_RDONLY | FA_HIDDEN | FA_SYSTEM);

    	while (!done) {
    		ListCount++;
        	FileName = ffb.ff_name;
        	ListFiles->Add(FileName);

            fsize = ffb.ff_fsize;

    		if (fsize == 0)
    			ksize = 0;
    		else if (fsize < 1536)
    			ksize= 1;
    		else {
    			ksize = fsize / 1024;
        		if (fsize % 1024 >= 512)
        		ksize++;
    		}

    		FileSize = "  " +  IntToStr(ksize) + "KB";

            if (ListFiles->Find(FileName, k)) {
            	ListSizes->Insert(k, FileSize);
            }
            else {
            	ListSizes->Add(FileSize);
            }

        	ListComments->Add("");
        	ListIndex->Add(" ");

        	done = findnext(&ffb);
    	}

        if (j == 0)
        	break;

        i += j;
        len -= j;
        Filter = ListFilter.SubString(i, len);
    }

    Target = 0;
    SelectCount = 0;
}
//---------------------------------------------------------------------------
void __fastcall TListForm::FormShow(TObject *Sender)
{
	int row;
    int width, height;

    if (ListCount == 0)
    	row = 1;
    else {
    	row = ListCount / ListColumn;
    	if (ListCount % ListColumn)
        	row++;
    }

    width = ListColumn * FaceWidth;
    height = row * FaceHeight;

    try {
        if (width < height) {
    		Image1->Picture->Bitmap->Width = width;
    		Image1->Picture->Bitmap->Height = height;

    		Image1->Width = width;
    		Image1->Height = height;
        }
        else {
    		Image1->Picture->Bitmap->Height = height;
            Image1->Picture->Bitmap->Width = width;

    		Image1->Height = height;
            Image1->Width = width;
        }
	}
    catch (...) {
    	Tag = -1;
        return;
    }

    Image1->Canvas->Pen->Color = clBtnFace;
    Image1->Canvas->Brush->Color = clBtnFace;
    Image1->Canvas->Brush->Style = bsSolid;
    Image1->Canvas->Rectangle(0, 0, Image1->Width, Image1->Height);

    width = ListColumn * FaceWidth;
    if (width > Screen->Width * 8 / 10)
    	width = Screen->Width * 8 / 10;
    width = width / FaceWidth * FaceWidth;
    if (width == 0)
    	width = 450;
    ClientWidth = width;

    height = row * FaceHeight;
    if (height > Screen->Height * 8 / 10)
    	height = Screen->Height * 8 / 10;
    height = height / FaceHeight * FaceHeight;
    if (height == 0)
    	height = 425;
    ClientHeight = height;

    Caption = ListDir + ListFilter;
}
//---------------------------------------------------------------------------
void __fastcall TListForm::FormCloseQuery(TObject *Sender, bool &CanClose)
{
	delete ListBitmap;
    delete ListSizes;
    delete ListFiles;
    delete ListComments;
    delete ListIndex;

    if (FrameForm->MDIChildCount == 1 && FrameForm->ROverView) {
            OverViewForm->Image1->Canvas->Brush->Color = clBtnFace;
        	OverViewForm->Image1->Canvas->FillRect(Rect(0, 0,
             OverViewForm->Image1->Width, OverViewForm->Image1->Height));
    }

    FrameForm->ListSetting = false;
    FrameForm->View1->Checked = false;
    FrameForm->SpeedButton3->Down = false;
    FrameForm->StatusBar1->Panels->Items[0]->Text = "";
}
//---------------------------------------------------------------------------
void __fastcall TListForm::FormClose(TObject *Sender, TCloseAction &Action)
{
	Action = caFree;
}
//---------------------------------------------------------------------------
void __fastcall TListForm::DispListFiles(void)
{
    int i;
    int rc;
    int	filter;
    int lx, ly;
    int Lx, Ly;
    int width, height, square;
    int	count, interval;
    int row;
    int ratio;
    bool chk;
    Graphics::TBitmap *bitmap;
    Graphics::TBitmap *bitmapq;
    AnsiString FileName;
    AnsiString Comment;

    if (ListCount == 0)
    	return;

    bitmap = new Graphics::TBitmap;
    bitmap->Canvas->Pen->Color = Color;			// ListForm ̃J[
    bitmap->Canvas->Brush->Color = Color;		// ListForm ̃J[
	bitmap->Canvas->Brush->Style = bsSolid;

    if (FrameForm->RListSquare == 1) {
    	bitmapq = new Graphics::TBitmap;
        bitmapq->Canvas->Pen->Color = Color;	// ListForm ̃J[
    	bitmapq->Canvas->Brush->Color = Color;	// ListForm ̃J[
		bitmapq->Canvas->Brush->Style = bsSolid;
    }

    filter = FrameForm->OpenListDialog->FilterIndex;

    if (filter > 6)		// *.exe *.dll *.*
        LIcon = new TIcon;

    TimerForm = new TTimerForm (Application);
    TimerForm->Show();
    TimerForm->Caption = "Making list of " + ListFilter;

    count = 0;
    interval = ListCount / 20;
    if (interval < 2) {
    	interval = 1;
        TimerForm->ProgressBar1->Step = 1;
        TimerForm->ProgressBar1->Max = (TProgressRange)ListCount;
    }
    else {
        TimerForm->ProgressBar1->Step = (TProgressRange)interval;
        TimerForm->ProgressBar1->Max = (TProgressRange)(ListCount - (ListCount % interval));
    }

    if (Tag == 0)
    	TimerForm->Button1->Enabled = true;
    else
    	TimerForm->Button1->Enabled = false;

    MakeFace();
    chk = false;					// ListBitmap ̑̍ق`

    for (i = 0; i < ListCount; i++) {
        if (FrameForm->TimerCancel)
        	break;

        ListBitmap->Canvas->Brush->Color = clBtnFace;
        ListBitmap->Canvas->Brush->Style = bsSolid;
    	ListBitmap->Canvas->Rectangle(2, 2, ListBitmap->Width - 3, ListBitmap->Height - 3);

        FileName = ListDir + ListFiles->Strings[i];
        rc = LoadFile(FileName, bitmap);

        if (rc == 0) {
            if (bitmap->Width <= ListSize && bitmap->Height <= ListSize) {
            	width = bitmap->Width;
                height = bitmap->Height;
                lx = (ListSize - width) / 2;
                ly = (ListSize - height) / 2;

                lx += ListSpace;
            	ly += ListSpace;

            	ListBitmap->Canvas->StretchDraw(Rect(lx, ly, lx + width,
            	 ly + height), bitmap);
            }
            else if (FrameForm->RListSquare == 0) {
            	if (bitmap->Width > bitmap->Height) {
            		width = ListSize;
                	height = ListSize * bitmap->Height / bitmap->Width;
                	lx = 0;
                	ly = (width - height) / 2;
            	}
            	else {
            		height = ListSize;
                	width = ListSize * bitmap->Width / bitmap->Height;
                	ly = 0;
                	lx = (height - width) / 2;
            	}

            	lx += ListSpace;
            	ly += ListSpace;

            	ListBitmap->Canvas->StretchDraw(Rect(lx, ly, lx + width,
             	 ly + height), bitmap);
            }
            else if (FrameForm->RListSquare == 1) {
            	if (bitmap->Width > bitmap->Height) {
            		square = bitmap->Height;
                	ly = 0;
                	lx = (bitmap->Width - square) / 2;
            	}
            	else {
                    square = bitmap->Width;
                	lx = 0;
                	ly = (bitmap->Height - square) / 2;
            	}

                bitmapq->Width = square;
                bitmapq->Height = square;

            	bitmapq->Canvas->CopyRect(Rect(0, 0, square, square),
                 bitmap->Canvas, Rect(lx, ly, lx + square, ly + square));

                if (square < ListSize) {
                	width = square;
                    lx = (ListSize - square) / 2;
                    ly = lx;
                }
                else {
                	width = ListSize;
                    lx = 0;
                    ly = 0;
                }

                lx += ListSpace;
            	ly += ListSpace;

                ListBitmap->Canvas->StretchDraw(Rect(lx, ly, lx + width,
                 ly + width), bitmapq);
            }
        }

        if (ListComments->Strings[i] == "") {
        	if (FrameForm->RListSizes == 0)
        		Comment = ListFiles->Strings[i] + ListSizes->Strings[i];
            else
                Comment = ListSizes->Strings[i];
        }
        else
        	Comment = ListComments->Strings[i];

        lx = ListSpace;
        ly = ListSpace + ListSize + 2 + 12 + FontDialog1->Font->Height;
        if (ly < 0)
        	ly = 0;
        ListBitmap->Canvas->Font = FontDialog1->Font;
        ListBitmap->Canvas->Brush->Style = bsClear;
        ListBitmap->Canvas->TextOut(lx, ly, Comment);

        width = ListBitmap->Width;
    	height = ListBitmap->Height;
        ListBitmap->Canvas->Pen->Color = clBlack;
    	ListBitmap->Canvas->MoveTo(0, height - 2);
    	ListBitmap->Canvas->LineTo(width - 2, height - 2);
    	ListBitmap->Canvas->LineTo(width - 2, 0);
    	ListBitmap->Canvas->MoveTo(1, height - 3);
    	ListBitmap->Canvas->LineTo(width - 3, height - 3);
    	ListBitmap->Canvas->LineTo(width - 3, 1);
        ListBitmap->Canvas->Pen->Color = clBtnFace;

        Lx = (i % ListColumn) * FaceWidth;
    	Ly = (i / ListColumn) * FaceHeight;

    	Image1->Canvas->StretchDraw(Rect(Lx, Ly, Lx + FaceWidth - 1,
    								Ly + FaceHeight - 1), ListBitmap);

        if (ListIndex->Strings[i] == "+") {
            chk = true;
        	DrawFrame(i, int(clBlue));
        }
        else {
        	if (chk) {
                chk = false;
            	DrawFrame(i, int(clBtnFace));
            }
        }

        Image1->Update();

        count++;
        if ((count % interval) == 0)
        	TimerForm->ProgressBar1->StepIt();

        ratio = count * 100 / ListCount;

        TimerForm->Caption = "  " + IntToStr(ratio) + " %";

        if (Tag == 0)
        	Application->ProcessMessages();
    }

    if (FrameForm->TimerCancel && Tag == 0) {
        ListCount = i;

        if (ListCount == 0)
    		row = 1;
    	else {
    		row = ListCount / ListColumn;
    		if (ListCount % ListColumn)
        		row++;
    	}

        width = ListColumn * FaceWidth;
    	height = row * FaceHeight;

        if (height < Image1->Height) {
        	bitmap->Width = width;
            bitmap->Height = height;

            bitmap->Canvas->CopyRect(Rect(0, 0, width, height),
             Image1->Picture->Bitmap->Canvas, Rect(0, 0, width, height));

            Image1->Picture->Bitmap->Height = height;
            Image1->Picture->Bitmap->Width = width;

    		Image1->Height = height;
            Image1->Width = width;

            Image1->Picture->Bitmap = bitmap;

            ClientWidth = width;
            ClientHeight = height;
        }
    }

    TimerForm->Close();

    if (filter > 6)		// *.exe *.dll *.*
        delete LIcon;

    if (FrameForm->RListSquare == 1)
    	delete bitmapq;

    delete bitmap;

    Tag = 1;
}
//---------------------------------------------------------------------------
int  __fastcall TListForm::LoadFile(const AnsiString FileName, Graphics::TBitmap *Bitmap)
{
	bool ft;
    int width, height;
    UINT bufsize;
    LPBYTE buf;
    AnsiString Ext;
    AnsiString ShortName;
    HMETAFILE hMetaFile;
    HENHMETAFILE hEnhMetaFile;
    ENHMETAHEADER EnhMetaHeader;
    SHFILEINFO shfi;
    DWORD ilist;
    RECT rect;
    HANDLE handle;
    FILE_INFO FT;
    WIN32_FIND_DATA ffd;
    DWORD size;

    Ext = ExtractFileExt(FileName);
    Ext = Ext.UpperCase();

    ft = false;

    if (Ext == ".BMP" || Ext == ".RLE") {
        ft = true;
        try {
    		Bitmap->LoadFromFile(FileName);
        }
        catch(...) {
        	ft = false;
        }
    }
    else if (Ext == ".JPG" || Ext == ".TIF") {
        ft = true;
        try {
    		ft = ImgKitFileType(FileName.c_str(), &FT);
        }
        catch(...) {
        	ft = false;
        }

        if (ft) {
            try {
        		handle = ImgKitFileLoad(FileName.c_str(), "", "");
            }
            catch(...) {
            	handle = 0;
            }

            if (handle != 0) {
                try {
            		Bitmap->Width = FT.Width;
                	Bitmap->Height = FT.Height;

                	ImgKitDisplayHdc(Bitmap->Canvas->Handle, handle, FT.Width, FT.Height, false);
                	ImgKitMemFree(handle);
                }
                catch(...) {
                	ft = false;
                }
            }
            else
            	ft = false;
        }
    }
    else if (Ext == ".ICO") {
    	ft = true;
        handle = ::LoadImage(HInstance, FileName.c_str(), IMAGE_ICON, 0, 0,
         LR_DEFAULTCOLOR | LR_LOADFROMFILE);

        if (handle == NULL) {
        	ft = false;
        }
        else {
        	::FindFirstFile(FileName.c_str(), &ffd);

            size = ffd.nFileSizeLow;
            if (size > 766)
            	size = 48;
            else if (size > 318)
            	size = 32;
            else
            	size = 16;

            width = (int)size;
            height = (int)size;

            Bitmap->Width = width;
            Bitmap->Height = height;

            Bitmap->Canvas->FillRect(Rect(0, 0, width, height));

            ::DrawIconEx(Bitmap->Canvas->Handle, 0, 0, handle, width, height,
             0, NULL, DI_NORMAL);

            ::CloseHandle(handle);
        }
    }
    else if (Ext == ".CUR") {
    	ft = true;
        handle = ::LoadImage(HInstance, FileName.c_str(), IMAGE_CURSOR, 0, 0,
         LR_DEFAULTCOLOR | LR_LOADFROMFILE);

         if (handle == NULL) {
        	ft = false;
        }
        else {
            width = 32;
            height = 32;

        	Bitmap->Width = width;
            Bitmap->Height = height;

            Bitmap->Canvas->FillRect(Rect(0, 0, width, height));

            ::DrawIconEx(Bitmap->Canvas->Handle, 0, 0, handle, width, height,
             0, NULL, DI_NORMAL);

            ::CloseHandle(handle);
        }
    }
    else if (Ext == ".WMF" || Ext == ".EMF") {
        ft = true;

        if (Ext == ".EMF") {
        	hEnhMetaFile = ::GetEnhMetaFile(FileName.c_str());
        }
        else {
        	ShortName = LongToShort(FileName);
        	hMetaFile = ::GetMetaFile(ShortName.c_str());
            bufsize = ::GetMetaFileBitsEx(hMetaFile, 0, NULL);
            buf = (LPBYTE)::GlobalAlloc(GPTR, bufsize);
            ::GetMetaFileBitsEx(hMetaFile, bufsize, buf);
            hEnhMetaFile = ::SetWinMetaFileBits(bufsize, buf,
             Bitmap->Canvas->Handle, NULL);
            ::GlobalFree(buf);
        }

        if (hEnhMetaFile == NULL) {
        	ft = false;
        }
        else {
        	::ZeroMemory(&EnhMetaHeader, sizeof(ENHMETAHEADER));

            ::GetEnhMetaFileHeader(hEnhMetaFile, sizeof(ENHMETAHEADER), &EnhMetaHeader);

            width = EnhMetaHeader.rclFrame.right - EnhMetaHeader.rclFrame.left + 1;
            height = EnhMetaHeader.rclFrame.bottom - EnhMetaHeader.rclFrame.top + 1;

            width = Screen->Width * width  / EnhMetaHeader.szlMillimeters.cx / 100;
            height = Screen->Height * height / EnhMetaHeader.szlMillimeters.cy / 100;

        	Bitmap->Width = width;
            Bitmap->Height = height;

            Bitmap->Canvas->FillRect(Rect(0, 0, width, height));

            rect.left = 0L;
            rect.right = (LONG)(width - 1);
            rect.top = 0L;
            rect.bottom = (LONG)(height - 1);

        	::PlayEnhMetaFile(Bitmap->Canvas->Handle, hEnhMetaFile, &rect);

            ::DeleteEnhMetaFile(hEnhMetaFile);
        }
    }
    else if (Ext == ".EXE" || Ext == ".DLL") {
        ft = true;
        ilist = ::SHGetFileInfo(FileName.c_str(), FILE_ATTRIBUTE_ARCHIVE, &shfi,
         sizeof(SHFILEINFO), SHGFI_ICON | SHGFI_LARGEICON);

        if (ilist == 0) {
        	ft = false;
        }
        else {
    		LIcon->ReleaseHandle();
    		LIcon->Handle = shfi.hIcon;

            width = LIcon->Width;
            height = LIcon->Height;

        	Bitmap->Width = width;
            Bitmap->Height = height;

            Bitmap->Canvas->FillRect(Rect(0, 0, width, height));

            Bitmap->Canvas->Draw(0, 0, LIcon);
        }
    }

    if (ft == false) {
		ft = FrameForm->SusieView(FileName, Bitmap);
    }

    if (ft == true)
    	return(0);
    else
    	return(-1);
}
//---------------------------------------------------------------------------
void __fastcall TListForm::MakeFace(void)
{
 	int width, height;

    width = ListBitmap->Width;
    height = ListBitmap->Height;

    ListBitmap->Canvas->Pen->Color = clWhite;
    ListBitmap->Canvas->Brush->Style = bsClear;
    ListBitmap->Canvas->Rectangle(0, 0, width - 1, height - 1);
    ListBitmap->Canvas->Rectangle(1, 1, width - 2, height - 2);

    ListBitmap->Canvas->Pen->Color = clBlack;
    ListBitmap->Canvas->MoveTo(0, height - 2);
    ListBitmap->Canvas->LineTo(width - 2, height - 2);
    ListBitmap->Canvas->LineTo(width - 2, 0);
    ListBitmap->Canvas->MoveTo(1, height - 3);
    ListBitmap->Canvas->LineTo(width - 3, height - 3);
    ListBitmap->Canvas->LineTo(width - 3, 1);

    ListBitmap->Canvas->Pen->Color = clBtnFace;
    ListBitmap->Canvas->Brush->Color = clBtnFace;
    ListBitmap->Canvas->Brush->Style = bsSolid;
    ListBitmap->Canvas->Rectangle(2, 2, width - 3, height - 3);
}
//---------------------------------------------------------------------------
void __fastcall TListForm::DrawFace(int i, int Color)
{
 	int Lx, Ly;

    Lx = (i % ListColumn) * FaceWidth;
    Ly = (i / ListColumn) * FaceHeight;

    Image1->Canvas->Pen->Color = (TColor)Color;
    Image1->Canvas->Brush->Color = (TColor)Color;
    Image1->Canvas->Brush->Style = bsSolid;

    Image1->Canvas->Rectangle(Lx + 2, Ly + 2, Lx + FaceWidth - 3, Ly + FaceHeight - 3);
}
//---------------------------------------------------------------------------
void __fastcall TListForm::DrawFrame(int i, int Color)
{
    int lx, ly;
 	int Lx, Ly;
    int width, height;
    AnsiString Comment;

    if (ListComments->Strings[i] == "") {
    	if (FrameForm->RListSizes == 0)
    		Comment = ListFiles->Strings[i] + ListSizes->Strings[i];
        else
    		Comment = ListSizes->Strings[i];
    }
    else
    	Comment = ListComments->Strings[i];

    Lx = (i % ListColumn) * FaceWidth;
    Ly = (i / ListColumn) * FaceHeight;

    ListBitmap->Canvas->CopyRect(Rect(0, 0, FaceWidth - 1, FaceHeight - 1),
     Image1->Canvas, Rect(Lx, Ly, Lx + FaceWidth - 1, Ly + FaceHeight - 1));

    ListBitmap->Canvas->Pen->Color = (TColor)Color;
    ListBitmap->Canvas->Brush->Color = (TColor)Color;
    ListBitmap->Canvas->Brush->Style = bsSolid;

    ListBitmap->Canvas->Rectangle(2, 2,
    						FaceWidth - 3, ListSpace);
    ListBitmap->Canvas->Rectangle(2, ListSpace + ListSize,
    						FaceWidth - 3, FaceHeight - 3);
    ListBitmap->Canvas->Rectangle(2, ListSpace,
    						ListSpace, ListSpace + ListSize);
    ListBitmap->Canvas->Rectangle(ListSpace + ListSize, ListSpace,
    						FaceWidth - 3, ListSpace + ListSize);

    lx = ListSpace;
    ly = ListSpace + ListSize + 2 + 12 + FontDialog1->Font->Height;
    if (ly < 0)
    	ly = 0;
    ListBitmap->Canvas->Font = FontDialog1->Font;
    ListBitmap->Canvas->Brush->Style = bsClear;
    ListBitmap->Canvas->TextOut(lx, ly, Comment);

    width = ListBitmap->Width;
    height = ListBitmap->Height;
    ListBitmap->Canvas->Pen->Color = clBlack;
    ListBitmap->Canvas->MoveTo(0, height - 2);
    ListBitmap->Canvas->LineTo(width - 2, height - 2);
    ListBitmap->Canvas->LineTo(width - 2, 0);
    ListBitmap->Canvas->MoveTo(1, height - 3);
    ListBitmap->Canvas->LineTo(width - 3, height - 3);
    ListBitmap->Canvas->LineTo(width - 3, 1);
    ListBitmap->Canvas->Pen->Color = clBtnFace;

    Image1->Canvas->CopyRect(Rect(Lx, Ly, Lx + FaceWidth - 1, Ly + FaceHeight - 1),
     ListBitmap->Canvas, Rect(0, 0, FaceWidth - 1, FaceHeight - 1));
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Open1Click(TObject *Sender)
{
    if (FrameForm->TimerOn)
    	return;

    FrameForm->OpenListFiles();
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Image1MouseDown(TObject *Sender, TMouseButton Button,
	TShiftState Shift, int X, int Y)
{
	int	i, j, k;
    int target;

    if (FrameForm->TimerOn)
    	return;

    if (Tag == 2) {
    	Tag = 1;
        return;
    }

    i = (Y / FaceHeight) * ListColumn + (X / FaceWidth);

    MouseTarget = i;

    if (i >= ListCount)
    	return;

    target = Target;
    Target = i;

    if (target < Target) {
    	i = target;
        j = Target;
    }
    else {
    	i = Target;
        j = target;
    }

    if (Button == mbLeft) {
    	if (Shift.Contains(ssShift)) {
    		for (k = i; k <= j; k++) {
            	if (ListIndex->Strings[k] == " ") {
                	ListIndex->Strings[k] = "+";
                    DrawFrame(k, int(clBlue));
                    SelectCount++;
                }
            }
        }
        else if (Shift.Contains(ssCtrl)) {
        	if (ListIndex->Strings[Target] == "+") {
            	ListIndex->Strings[Target] = " ";
                DrawFrame(Target, int(clBtnFace));
                SelectCount--;
            }
            else {
            	ListIndex->Strings[Target] = "+";
                DrawFrame(Target, int(clBlue));
                SelectCount++;
            }
        }
        else {
        	for (k = 0; k < ListCount; k++) {
            	if (k == Target) {
                	if (ListIndex->Strings[k] == " ") {
                    	ListIndex->Strings[k] = "+";
                        DrawFrame(k, int(clBlue));
                    }
                }
                else {
                	if (ListIndex->Strings[k] == "+") {
                    	ListIndex->Strings[k] = " ";
                        DrawFrame(k, int(clBtnFace));
                    }
                }
            }
            SelectCount = 1;
        }
    }
    else if (Button == mbRight) {
    	if (ListIndex->Strings[Target] == " ") {
        	ListIndex->Strings[Target] = "+";
            DrawFrame(Target, int(clBlue));
            SelectCount++;
        }
    }

    if (FrameForm->ROverView)
    	OverViewForm->DrawListView();
}
//---------------------------------------------------------------------------
void __fastcall TListForm::FormActivate(TObject *Sender)
{
    if (FrameForm->TimerOn == false && FrameForm->MDIChildCount > 1)
    	FrameForm->DeactiveSpeedBar();

    if (FrameForm->TextSetting)
    	ViewForm->Text1Click(Sender);
    else if (FrameForm->AreaSetting)
    	ViewForm->Areasetting1Click(Sender);

    FrameForm->StatusBar1->Panels->Items[0]->Text = IntToStr(Image1->Width)
     + " * " + IntToStr(Image1->Height);
    FrameForm->StatusBar1->Panels->Items[1]->Text = "";
    FrameForm->StatusBar1->Panels->Items[4]->Text = "";

    if (FrameForm->ROverView)
    	OverViewForm->DrawListView();

    if (FrameForm->RHistgram)
    	HistgramForm->ClearHistgram();
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Image1DblClick(TObject *Sender)
{
    AnsiString FileName;

    if (FrameForm->TimerOn)
    	return;

    FileName = ListDir + ListFiles->Strings[Target];
    FrameForm->OpenListFile(FileName);
    Tag = 2;
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Clear1Click(TObject *Sender)
{
	int i;

    if (ListCount == 0)
    	return;

    for (i = 0; i < ListCount; i++) {
    	if (ListIndex->Strings[i] == "+") {
        	ListIndex->Strings[i] = " ";
            DrawFrame(i, int(clBtnFace));
        }
    }

    SelectCount = 0;

    if (FrameForm->ROverView)
    	OverViewForm->DrawListView();
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Escape1Click(TObject *Sender)
{
	return;
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Delete1Click(TObject *Sender)
{
	int i, Result, len, ldlen;
    char *ptr;
    char *DelFiles;
    AnsiString DelFile;
    SHFILEOPSTRUCT SHFileOp;

    if (SelectCount == 0)
    	return;

    ldlen = ListDir.Length();

    for (i = 0, len = 0; i < ListCount; i++) {
    	if (ListIndex->Strings[i] == "+") {
        	len += ldlen;
            len += ListFiles->Strings[i].Length();
            len++;
        }
    }
    len++;

    DelFiles = (char *)calloc(len, sizeof(char));
    if (DelFiles == NULL) {
    	ShowMessage("słB폜𑱍so܂B");
        return;
    }

    for (i = 0, ptr = DelFiles; i < ListCount; i++) {
    	if (ListIndex->Strings[i] == "+") {
        	DelFile = ListDir + ListFiles->Strings[i];
            strcpy(ptr, DelFile.c_str());
            ptr = strchr(ptr, '\0');
            ptr++;
        }
        *ptr = '\0';
    }

    SHFileOp.hwnd = FrameForm->Handle;
    SHFileOp.wFunc = FO_DELETE;
    SHFileOp.pFrom = DelFiles;
    SHFileOp.pTo = NULL;
    SHFileOp.fFlags = FOF_ALLOWUNDO;
    SHFileOp.fAnyOperationsAborted = NULL;
    SHFileOp.hNameMappings = NULL;
    SHFileOp.lpszProgressTitle = NULL;

    Result = SHFileOperation(&SHFileOp);

    free(DelFiles);

    if (Result == 0 && SHFileOp.fAnyOperationsAborted == 0) {
    	len = SelectCount;
        for (i = 0; i < ListCount; i++) {
        	if (ListIndex->Strings[i] == "+")
            	ListIndex->Strings[i] = "-";
        }
        RemakeList();

        ShowMessage(IntToStr(len) + " ̃t@C폜܂B");
    }
}
//---------------------------------------------------------------------------
void __fastcall TListForm::RemakeList(void)
{
	int i, j, row, count;
    int width, height;
    int slx, sly, dlx, dly;
    Graphics::TBitmap *bitmap;

    if (WindowState == wsMaximized)
    	WindowState = wsNormal;

    for (i = 0, count = 0; i < ListCount; i++) {
    	if (ListIndex->Strings[i] == "-")
        	count++;
        else if (ListIndex->Strings[i] == "+") {
        	ListIndex->Strings[i] = " ";
            DrawFrame(i, (int)clBtnFace);
        }
    }

    Target = 0;
    SelectCount = 0;

    if (count == 0)
    	return;

    j = ListCount - count;

    if (j == 0)
    	row = 1;
    else {
    	row = j / ListColumn;
        if (j % ListColumn)
        	row++;
    }

    width = ListColumn * FaceWidth;
    height = row * FaceHeight;

    bitmap = new Graphics::TBitmap;

    bitmap->Width = width;
    bitmap->Height = height;

    bitmap->Canvas->Pen->Color = clBtnFace;
    bitmap->Canvas->Brush->Color = clBtnFace;
    bitmap->Canvas->Brush->Style = bsSolid;
    bitmap->Canvas->Rectangle(0, 0, bitmap->Width, bitmap->Height);

    for (i = 0, j = 0; i < ListCount; i++) {
    	if (ListIndex->Strings[i] == " ") {
        	slx = (i % ListColumn) * FaceWidth;
            sly = (i / ListColumn) * FaceHeight;
            dlx = (j % ListColumn) * FaceWidth;
            dly = (j / ListColumn) * FaceHeight;
            bitmap->Canvas->CopyRect(Rect(dlx, dly, dlx + FaceWidth - 1,
             dly + FaceHeight - 1), Image1->Canvas, Rect(slx, sly,
              slx + FaceWidth - 1, sly + FaceHeight - 1));
            j++;
        }
    }

    Image1->Picture->Bitmap->Width = width;
    Image1->Picture->Bitmap->Height = height;

    Image1->Width = width;
    Image1->Height = height;

    width = ListColumn * FaceWidth;
    if (width > Screen->Width * 8 / 10)
    	width = Screen->Width * 8 / 10;
    width = width / FaceWidth * FaceWidth;
    if (width == 0)
    	width = 450;
    ClientWidth = width;

    height = row * FaceHeight;
    if (height > Screen->Height * 8 / 10)
    	height = Screen->Height * 8 / 10;
    height = height / FaceHeight * FaceHeight;
    if (height == 0)
    	height = 425;
    ClientHeight = height;

    Image1->AutoSize = false;
    Image1->Stretch = true;

    Image1->Picture->Bitmap = bitmap;

//    Image1->Canvas->StretchDraw(Rect(0, 0, width - 1, height - 1), bitmap);

    delete bitmap;

    for (i = 0; i < ListCount; i++) {
    	if (ListIndex->Strings[i] == "-") {
        	ListFiles->Delete(i);
            ListSizes->Delete(i);
            ListComments->Delete(i);
            ListIndex->Delete(i);
            ListCount--;
            i--;
        }
    }

    if (FrameForm->ROverView)
    	OverViewForm->DrawListView();
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Rename1Click(TObject *Sender)
{
	int rc;
    bool ri;
    AnsiString OldName;
    AnsiString NewName;

    if (ListCount == 0 || MouseTarget >= ListCount)
    	return;

    OldName = ListFiles->Strings[Target];
    NewName = OldName;

    ri = InputQuery("t@C̕ύX - " + OldName, "Vt@C͂ĉB",
    				NewName);

    if (ri == false || NewName == OldName)
    	return;

    OldName = ListDir + NewName;
    NewName = OldName;
    OldName = ListDir + ListFiles->Strings[Target];

    rc = rename(OldName.c_str(), NewName.c_str());

    if (rc == -1)
    	ShowMessage(ListFiles->Strings[Target] + " ́At@CύXo܂B");
    else {
    	ListFiles->Strings[Target] = ExtractFileName(NewName);
        DrawFrame(Target, (int)clBlue);
        if (FrameForm->ROverView)
    		OverViewForm->DrawListView();
    }
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Copy1Click(TObject *Sender)
{
	int i, count, done;
    int rc, Result;
    struct ffblk ffb;
    AnsiString SrcFile;
    AnsiString DestDir, DestFile;

    if (SelectCount == 0)
    	return;

	DirectoryListForm = new TDirectoryListForm (this);

    DirectoryListForm->Caption = "IĂ̧ق̺߰͂ĉB";

    DirectoryListForm->Label1->Caption = "̫߰ : ";

    Result = DirectoryListForm->ShowModal();

    DestDir = DirectoryListForm->Edit1->Text;
    if (DestDir.Length() > 3)
    	DestDir += "\\";

    delete DirectoryListForm;

    if (Result != mrOk)
    	return;
    else if (ListDir == DestDir) {
    	ShowMessage("tH_ɂ́ARs[o܂B");
    	return;
    }

    for (i = 0, count = 0; i < ListCount; i++) {
    	if (ListIndex->Strings[i] == "+") {
        	SrcFile = ListDir + ListFiles->Strings[i];
            DestFile = DestDir + ListFiles->Strings[i];

            done = findfirst(DestFile.c_str(), &ffb,
             FA_ARCH | FA_RDONLY | FA_HIDDEN | FA_SYSTEM | FA_DIREC);

            if (!done) {
            	rc = MessageDlg(ListFiles->Strings[i] + " ́ARs[ɑ݂܂B㏑܂H",
                 mtConfirmation, TMsgDlgButtons() << mbYes << mbNo, 0);

                if (rc == mrYes)
                	done = -1;
            }

            if (done) {
            	rc = CopyFile(SrcFile.c_str(), DestFile.c_str(), 0);
                if (rc == 0)
                	ShowMessage(ListFiles->Strings[i] + " ́ARs[o܂B");
                else
                	count++;
            }
        }
    }

    ShowMessage(IntToStr(count) + " ̃t@CRs[܂B");
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Move1Click(TObject *Sender)
{
	int i, count, done;
    int rc, Result;
    struct ffblk ffb;
    AnsiString SrcFile;
    AnsiString DestDir, DestFile;

    if (SelectCount == 0)
    	return;

	DirectoryListForm = new TDirectoryListForm (this);

    DirectoryListForm->Caption = "IĂ̧ق̈ړ͂ĉB";

    DirectoryListForm->Label1->Caption = "ړ̫ : ";

    Result = DirectoryListForm->ShowModal();

    DestDir = DirectoryListForm->Edit1->Text;
    if (DestDir.Length() > 3)
    	DestDir += "\\";

    delete DirectoryListForm;

    if (Result != mrOk)
    	return;
    else if (ListDir == DestDir) {
    	ShowMessage("tH_ɂ́Aړo܂B");
    	return;
    }

    if (ListDir.SubString(1, 1) != DestDir.SubString(1, 1)) {
    	ShowMessage("قȂhCuԂł̃t@C̈ړ́Ao܂B");
        return;
    }

    for (i = 0, count = 0; i < ListCount; i++) {
    	if (ListIndex->Strings[i] == "+") {
        	SrcFile = ListDir + ListFiles->Strings[i];
            DestFile = DestDir + ListFiles->Strings[i];

            done = findfirst(DestFile.c_str(), &ffb,
             FA_ARCH | FA_RDONLY | FA_HIDDEN | FA_SYSTEM | FA_DIREC);

            if (!done)
            	ShowMessage(ListFiles->Strings[i] + " Ɠ̃t@Cړɑ݂܂̂ňړ͏o܂B");
            else {
            	rc = MoveFile(SrcFile.c_str(), DestFile.c_str());
                if (rc == 0)
                	ShowMessage(ListFiles->Strings[i] + " ́Aړo܂B");
                else {
                    ListIndex->Strings[i] = "-";
                	count++;
                }
            }
        }
    }

    if (count > 0)
    	RemakeList();

    ShowMessage(IntToStr(count) + " ̃t@Cړ܂B");

}
//---------------------------------------------------------------------------
void __fastcall TListForm::Savelist1Click(TObject *Sender)
{
    int rc, left, top;

    if (ListCount == 0)
    	return;

    FrameForm->TimerOn = true;

    rc = MessageDlg("̃t@Cꗗʂ摜t@CƂĕۑ܂B",
     mtConfirmation, TMsgDlgButtons() << mbOK << mbCancel, 0);

    if (rc != mrOk)
    	return;

    if (WindowState == wsMaximized)
    	WindowState = wsNormal;

    ClearFaces();

    left = Screen->Width / 2 - 76 - FrameForm->Left;
    top = Screen->Height / 2 - 130 - FrameForm->Top;

	ViewForm = new TViewForm (this);

    ViewForm->Left = left;
    ViewForm->Top = top;
    ViewForm->Image1 = Image1;
    ViewForm->Save1Click(Sender);

    delete ViewForm;

    RestoreFaces();

    FrameForm->TimerOn = false;
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Close1Click(TObject *Sender)
{
	int i;
    AnsiString FileName;

    if (SelectCount == 0)
    	return;

    for (i = 0; i < ListCount; i++) {
    	if (ListIndex->Strings[i] == "+")
        	ListIndex->Strings[i] = "-";
    }

    RemakeList();
    FrameForm->StatusBar1->Panels->Items[0]->Text = IntToStr(Image1->Width)
     + " * " + IntToStr(Image1->Height);
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Size1Click(TObject *Sender)
{
    int i, handle, Result;
    int size, width;
    int OListSize, OMaxImageWidth, OListColumn;
    int OFaceWidth, OFaceHeight;
    AnsiString	TempFile;

    if (ListCount == 0)
    	return;

    if (WindowState == wsMaximized)
    	WindowState = wsNormal;

    ListSizeForm = new TListSizeForm (this);

    Result = ListSizeForm->ShowModal();

    try {
    	size = StrToInt(ListSizeForm->Edit1->Text) - (ListSpace * 2);
    	width = StrToInt(ListSizeForm->Edit2->Text);
    }
    catch(...) {
    	delete ListSizeForm;
        if (Result == mrOk)
        	ShowMessage("͒l܂B");
        return;
    }

    delete ListSizeForm;

    if (Result != mrOk || ((size == ListSize) && (width == MaxImageWidth)))
    	return;

    if (size < 40)
    	size = 40;
    else if (size > 5000)
    	size = 5000;

    if (width < 40 + ListSpace * 2)
    	width = 40 + ListSpace * 2;
    else if (width > 5000 + ListSpace * 2)
    	width = 5000 + ListSpace * 2;

    if (width < size + ListSpace * 2)
    	width = size + ListSpace * 2;

    OListSize = ListSize;
    OMaxImageWidth = MaxImageWidth;
    OListColumn = ListColumn;
    OFaceWidth = FaceWidth;
    OFaceHeight = FaceHeight;

    FrameForm->RListSize = size;
    MaxImageWidth = width;
    FrameForm->RMaxImageWidth = width;

    ListSize = size;
    ListColumn = MaxImageWidth / FaceWidth;

    ListBitmap->Width = FaceWidth;
    ListBitmap->Height = FaceHeight;

    Image1->AutoSize = false;
    Image1->Stretch = true;

    if (ListSize > OListSize) {
    	for (i = 0; i < 100; i++) {
			TempFile = FrameForm->SystemRoot + "\\Tempiv" + IntToStr(i) + ".bmp";
            handle = open(TempFile.c_str(), O_RDONLY | O_BINARY);
            if (handle == -1)
            	break;
            else
            	close(handle);
        }
        if (i < 100) {
        	Image1->Picture->Bitmap->SaveToFile(TempFile);
            i = -1;
        }
        else
        	i = 0;
    }
    else
    	i = 0;

    FormShow(Sender);
    if (Tag == -1) {		// Error
        ListSize = OListSize;
        FrameForm->RListSize = OListSize;
    	MaxImageWidth = OMaxImageWidth;
        FrameForm->RMaxImageWidth = OMaxImageWidth;
    	ListColumn = OListColumn;
    	ListBitmap->Width = OFaceWidth;
    	ListBitmap->Height = OFaceHeight;
        FormShow(Sender);
    }

    if (Tag == -1 && i == -1) {
    	Image1->Picture->Bitmap->LoadFromFile(TempFile);
        ShowMessage("TlC̃TCY傫܂B");
    }
    else {
        if (Tag == -1)
    		ShowMessage("TlC̃TCY傫܂B");
    	DispListFiles();
        if (FrameForm->ROverView)
        	OverViewForm->DrawListView();
    }

    if (i == -1)
    	unlink(TempFile.c_str());

    FrameForm->StatusBar1->Panels->Items[0]->Text = IntToStr(Image1->Width)
     + " * " + IntToStr(Image1->Height);

    Tag = 1;
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Preview1Click(TObject *Sender)
{
    int left, top;

    if (ListCount == 0)
    	return;

    FrameForm->TimerOn = true;

    if (WindowState == wsMaximized)
    	WindowState = wsNormal;

    ClearFaces();

    left = Screen->Width / 2 - 76 - FrameForm->Left;
    top = Screen->Height / 2 - 130 - FrameForm->Top;

	ViewForm = new TViewForm (this);

    ViewForm->Left = left;
    ViewForm->Top = top;
    ViewForm->Image1 = Image1;
    ViewForm->Preview1Click(Sender);

    delete ViewForm;

    RestoreFaces();

    FrameForm->TimerOn = false;
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Print2Click(TObject *Sender)
{
	int left, top;

    if (ListCount == 0)
    	return;

    FrameForm->TimerOn = true;

    if (WindowState == wsMaximized)
    	WindowState = wsNormal;

    ClearFaces();

    left = Screen->Width / 2 - 76 - FrameForm->Left;
    top = Screen->Height / 2 - 130 - FrameForm->Top;

	ViewForm = new TViewForm (this);

    ViewForm->Left = left;
    ViewForm->Top = top;
    ViewForm->Image1 = Image1;
    ViewForm->Print2Click(Sender);

    delete ViewForm;

    RestoreFaces();

    FrameForm->TimerOn = false;
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Setup1Click(TObject *Sender)
{
	int left, top;

    if (ListCount == 0)
    	return;

    FrameForm->TimerOn = true;

    if (WindowState == wsMaximized)
    	WindowState = wsNormal;

    ClearFaces();

    left = Screen->Width / 2 - 76 - FrameForm->Left;
    top = Screen->Height / 2 - 130 - FrameForm->Top;

	ViewForm = new TViewForm (this);

    ViewForm->Left = left;
    ViewForm->Top = top;
    ViewForm->Image1 = Image1;
    ViewForm->Setup1Click(Sender);

    delete ViewForm;

    RestoreFaces();

    FrameForm->TimerOn = false;
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Selectall1Click(TObject *Sender)
{
	int i;

    if (ListCount == 0)
    	return;

    for (i = 0; i < ListCount; i++) {
    	if (ListIndex->Strings[i] == " ") {
        	ListIndex->Strings[i] = "+";
            DrawFrame(i, int(clBlue));
        }
    }

    SelectCount = ListCount;

    if (FrameForm->ROverView)
    	OverViewForm->DrawListView();
}
//---------------------------------------------------------------------------
void __fastcall TListForm::FormKeyDown(TObject *Sender, WORD &Key,
	TShiftState Shift)
{
    if (FrameForm->TimerOn)
    	return;

    if (Shift.Contains(ssCtrl)) {
		if (Key == (WORD)'A')
    		Selectall1Click(Sender);
    	else if (Key == (WORD)'R')
    		Revers1Click(Sender);
        else if (Key == (WORD)'C')
    		Clear1Click(Sender);
        else if (Key == (WORD)'F')
    		DispSizes();

        if (FrameForm->ROverView)
        	OverViewForm->DrawListView();
    }
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Revers1Click(TObject *Sender)
{
	int i;

    if (ListCount == 0)
    	return;

    for (i = 0; i < ListCount; i++) {
    	if (ListIndex->Strings[i] == " ") {
        	ListIndex->Strings[i] = "+";
            DrawFrame(i, int(clBlue));
        }
        else if (ListIndex->Strings[i] == "+") {
        	ListIndex->Strings[i] = " ";
            DrawFrame(i, int(clBtnFace));
        }
    }

    SelectCount = ListCount - SelectCount;

    if (FrameForm->ROverView)
    	OverViewForm->DrawListView();
}
//---------------------------------------------------------------------------
void __fastcall TListForm::DispListFiles2(void)
{
    int i;
    int rc;
    int	filter;
    int lx, ly;
    int Lx, Ly;
    int width, height, square;
    int	count, interval;
    int ratio;
    Graphics::TBitmap *bitmap;
    Graphics::TBitmap *bitmapq;
    AnsiString FileName;
    AnsiString Comment;

    if (SelectCount == 0)
    	return;

    bitmap = new Graphics::TBitmap;
    bitmap->Canvas->Pen->Color = Color;			// ListForm ̃J[
    bitmap->Canvas->Brush->Color = Color;		// ListForm ̃J[
	bitmap->Canvas->Brush->Style = bsSolid;

    if (FrameForm->RListSquare == 1) {
    	bitmapq = new Graphics::TBitmap;
        bitmapq->Canvas->Pen->Color = Color;	// ListForm ̃J[
    	bitmapq->Canvas->Brush->Color = Color;	// ListForm ̃J[
		bitmapq->Canvas->Brush->Style = bsSolid;
    }

    filter = FrameForm->OpenListDialog->FilterIndex;

    if (filter > 6)		// *.exe *.dll *.*
        LIcon = new TIcon;

    if (SelectCount > 1) {
    	TimerForm = new TTimerForm (Application);
    	TimerForm->Show();
        TimerForm->Caption = "Making list of " + ListFilter;

        count = 0;
    	interval = SelectCount / 20;
    	if (interval < 2) {
    		interval = 1;
        	TimerForm->ProgressBar1->Step = 1;
        	TimerForm->ProgressBar1->Max = (TProgressRange)SelectCount;
    	}
        else {
        	TimerForm->ProgressBar1->Step = (TProgressRange)interval;
        	TimerForm->ProgressBar1->Max = (TProgressRange)(SelectCount - (SelectCount % interval));
    	}

        TimerForm->Button1->Enabled = false;
    }

    MakeFace();					// ListBitmap ̑̍ق`

    for (i = 0; i < ListCount; i++) {
        if (ListIndex->Strings[i] == "+") {
        	ListBitmap->Canvas->Brush->Color = clBtnFace;
       	 	ListBitmap->Canvas->Brush->Style = bsSolid;
    		ListBitmap->Canvas->Rectangle(2, 2, ListBitmap->Width - 3, ListBitmap->Height - 3);

        	width = ListBitmap->Width;
    		height = ListBitmap->Height;
        	ListBitmap->Canvas->Pen->Color = clBlack;
    		ListBitmap->Canvas->MoveTo(0, height - 1);
    		ListBitmap->Canvas->LineTo(width - 1, height - 1);
    		ListBitmap->Canvas->LineTo(width - 1, 0);
    		ListBitmap->Canvas->MoveTo(1, height - 2);
    		ListBitmap->Canvas->LineTo(width - 2, height - 2);
    		ListBitmap->Canvas->LineTo(width - 2, 1);
        	ListBitmap->Canvas->Pen->Color = clBtnFace;

        	FileName = ListDir + ListFiles->Strings[i];
        	rc = LoadFile(FileName, bitmap);

        	if (rc == 0) {
            	if (bitmap->Width <= ListSize && bitmap->Height <= ListSize) {
            		width = bitmap->Width;
                	height = bitmap->Height;
                	lx = (ListSize - width) / 2;
                	ly = (ListSize - height) / 2;

                	lx += ListSpace;
            		ly += ListSpace;

            		ListBitmap->Canvas->StretchDraw(Rect(lx, ly, lx + width,
            		 ly + height), bitmap);
            	}
            	else if (FrameForm->RListSquare == 0) {
            		if (bitmap->Width > bitmap->Height) {
            			width = ListSize;
                		height = ListSize * bitmap->Height / bitmap->Width;
                		lx = 0;
                		ly = (width - height) / 2;
            		}
            		else {
            			height = ListSize;
                		width = ListSize * bitmap->Width / bitmap->Height;
                		ly = 0;
                		lx = (height - width) / 2;
            		}

            		lx += ListSpace;
            		ly += ListSpace;

            		ListBitmap->Canvas->StretchDraw(Rect(lx, ly, lx + width,
             		 ly + height), bitmap);
            	}
            	else if (FrameForm->RListSquare == 1) {
            		if (bitmap->Width > bitmap->Height) {
            			square = bitmap->Height;
                		ly = 0;
                		lx = (bitmap->Width - square) / 2;
            		}
            		else {
                    	square = bitmap->Width;
                		lx = 0;
                		ly = (bitmap->Height - square) / 2;
            		}

                	bitmapq->Width = square;
                	bitmapq->Height = square;

            		bitmapq->Canvas->CopyRect(Rect(0, 0, square, square),
                	 bitmap->Canvas, Rect(lx, ly, lx + square, ly + square));

                    if (square < ListSize) {
                		width = square;
                    	lx = (ListSize - square) / 2;
                    	ly = lx;
                	}
                	else {
                		width = ListSize;
                    	lx = 0;
                    	ly = 0;
                	}

                	lx += ListSpace;
            		ly += ListSpace;

                	ListBitmap->Canvas->StretchDraw(Rect(lx, ly, lx + width,
                	 ly + width), bitmapq);
            	}
        	}

            if (ListComments->Strings[i] == "") {
            	if (FrameForm->RListSizes == 0)
    				Comment = ListFiles->Strings[i] + ListSizes->Strings[i];
        		else
            		Comment = ListSizes->Strings[i];
            }
        	else
        		Comment = ListComments->Strings[i];

        	lx = ListSpace;
        	ly = ListSpace + ListSize + 2 + 12 + FontDialog1->Font->Height;
        	if (ly < 0)
        		ly = 0;
        	ListBitmap->Canvas->Font = FontDialog1->Font;
        	ListBitmap->Canvas->Brush->Style = bsClear;
        	ListBitmap->Canvas->TextOut(lx, ly, Comment);

        	Lx = (i % ListColumn) * FaceWidth;
    		Ly = (i / ListColumn) * FaceHeight;

    		Image1->Canvas->StretchDraw(Rect(Lx, Ly, Lx + FaceWidth - 1,
    								Ly + FaceHeight - 1), ListBitmap);

            DrawFrame(i, int(clBlue));

        	Image1->Update();

            if (SelectCount > 1) {
            	count++;
        		if ((count % interval) == 0)
        			TimerForm->ProgressBar1->StepIt();

                ratio = count * 100 / SelectCount;
        		TimerForm->Caption = "  " + IntToStr(ratio) + " %";
    		}
        }
    }

    if (SelectCount > 1)
    	TimerForm->Close();

    if (filter > 6)		// *.exe *.dll *.*
        delete LIcon;

    if (FrameForm->RListSquare == 1)
    	delete bitmapq;

    delete bitmap;
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Comment1Click(TObject *Sender)
{
    int i;
    AnsiString Comment;

    if (SelectCount == 0)
    	return;

    if (WindowState == wsMaximized)
    	WindowState = wsNormal;

    for (i = 0; i < ListCount; i++) {
    	if (ListIndex->Strings[i] == "+") {
        	Comment = ListComments->Strings[i];
            if (Comment != "")
            	break;
        }
    }

    if (InputQuery("Rg̓", "݂̑It@CɃRgt܂B",
    				Comment)) {
    	for (i = 0; i < ListCount; i++) {
    		if (ListIndex->Strings[i] == "+") {
        		ListComments->Strings[i] = Comment;
            }
    	}

//    	FormShow(Sender);
    	DispListFiles2();
        if (FrameForm->ROverView)
        	OverViewForm->DrawListView();
    }
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Font1Click(TObject *Sender)
{
    if (ListCount == 0)
    	return;

    if (WindowState == wsMaximized)
    	WindowState = wsNormal;

    ShowMessage("Rg̃tHgw肵܂B");

	if (FontDialog1->Execute()) {
//    	FormShow(Sender);
    	DispListFiles();
        if (FrameForm->ROverView)
        	OverViewForm->DrawListView();
    }
}
//---------------------------------------------------------------------------
void __fastcall TListForm::Square1Click(TObject *Sender)
{
	if (WindowState == wsMaximized)
    	WindowState = wsNormal;

	if (FrameForm->RListSquare == 0) {
    	FrameForm->RListSquare = 1;
        Square1->Checked = true;
    }
    else if (FrameForm->RListSquare == 1) {
    	FrameForm->RListSquare = 0;
        Square1->Checked = false;
    }

    DispListFiles();
    if (FrameForm->ROverView)
    	OverViewForm->DrawListView();
}
//---------------------------------------------------------------------------
void __fastcall TListForm::ClearFaces(void)
{
	int i;

    if (ListCount == 0)
    	return;

    for (i = 0; i < ListCount; i++) {
    	if (ListIndex->Strings[i] == "+") {
            DrawFrame(i, int(clBtnFace));
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TListForm::RestoreFaces(void)
{
	int i;

    if (ListCount == 0)
    	return;

    for (i = 0; i < ListCount; i++) {
    	if (ListIndex->Strings[i] == "+") {
            DrawFrame(i, int(clBlue));
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TListForm::CopytoClipboard11Click(TObject *Sender)
{
    ClearFaces();

    Clipboard()->Assign(Image1->Picture->Bitmap);

    RestoreFaces();
}
//---------------------------------------------------------------------------
AnsiString __fastcall TListForm::LongToShort(const AnsiString PathName)
{
	AnsiString ShortName;
    char longname[MAX_PATH];
    char temp[16];
    char *ptr;
    WIN32_FIND_DATA ffd;

    ShortName = "";
    strcpy(longname, PathName.c_str());

    if (FindFirstFile(longname, &ffd) == INVALID_HANDLE_VALUE) {
    	return(ShortName);
    }

    while (1) {
    	ptr = strrchr(longname, '\\');

        if (ptr != NULL) {
            strcpy(temp, "\\");
        	if (ffd.cAlternateFileName[0] == '\0')
            	strcat(temp, ffd.cFileName);
            else
            	strcat(temp, ffd.cAlternateFileName);
        	ShortName.Insert(temp, 1);
        }
        else {
        	ShortName.Insert(longname, 1);
            break;
        }

        *ptr = '\0';
        FindFirstFile(longname, &ffd);
    }

    return(ShortName);
}
//---------------------------------------------------------------------------
void __fastcall TListForm::FormDeactivate(TObject *Sender)
{
    if (FrameForm->TimerOn == false && FrameForm->MDIChildCount > 1)
    	FrameForm->ActiveSpeedBar();
}
//---------------------------------------------------------------------------
void __fastcall TListForm::DispSizes(void)
{
	if (WindowState == wsMaximized)
    	WindowState = wsNormal;

	if (FrameForm->RListSizes == 0)
    	FrameForm->RListSizes = 1;
    else if (FrameForm->RListSizes == 1)
    	FrameForm->RListSizes = 0;

    DispListFiles();
    if (FrameForm->ROverView)
    	OverViewForm->DrawListView();
}
//---------------------------------------------------------------------------
